#include "CompressorPlug.h"

static PlugInitInfo info;

/******** CompressorPlugin Implementation *********/

BEGIN_EVENT_TABLE(CompressorPlugin, wxWindow)
  EVT_COMMAND_SCROLL(Compressor_Target, CompressorPlugin::OnTarget)
  EVT_COMMAND_SCROLL(Compressor_Res, CompressorPlugin::OnResonance)
  EVT_PAINT(CompressorPlugin::OnPaint)
END_EVENT_TABLE()

CompressorPlugin::CompressorPlugin(PlugStartInfo &startinfo, PlugInitInfo *initinfo)
  : Plugin(startinfo, initinfo)
{
  Init();

  wxImage *tr_bg = 
    new wxImage(string(GetDataDir() + string(IMG_CP_BG)).c_str(), wxBITMAP_TYPE_PNG);
  if (tr_bg)
    TpBmp = new wxBitmap(tr_bg);
  
  bmp = new wxBitmap(string(GetDataDir() + string(IMG_CP_BMP)).c_str(), wxBITMAP_TYPE_BMP); 

  img_bg = new wxImage(string(GetDataDir() + string(IMG_CP_FADER_BG)).c_str(), wxBITMAP_TYPE_PNG );
  img_fg = new wxImage(string(GetDataDir() + string(IMG_CP_FADER_FG)).c_str(), wxBITMAP_TYPE_PNG );
  
  TargetFader = new 
    FaderCtrl(this, Compressor_Target, img_bg, img_fg, 0, 100, 100,
	      wxPoint(18, 8)/*wxPoint(GetSize().x / 2, 10)*/, wxSize(22,78));
  ResFader = new 
    FaderCtrl(this, Compressor_Res, img_bg, img_fg, 0, 100, 0,
	      wxPoint(142, 8)/*wxPoint(GetSize().x / 2 + 40, 10)*/, wxSize(22,78));
  
  //  Target = 1.f;
  //Res = 0.f;

  SetBackgroundColour(wxColour(237, 237, 237));
}


void CompressorPlugin::Init()
{
  /*
  SamplingRate = 44100.f;
  Threshold = 0.01;          // threshold to unity (0...1)
  Slope = 2;              // slope to unity
  LookAhead = 3 * 1e-3;                // lookahead time to seconds
  WindowTime = 1 * 1e-3;               // window time to seconds
  AttackTime = 0.1 * 1e-3;               // attack time to seconds
  ReleaseTime = 300 * 1e-3;               // release time to seconds

  // attack and release "per sample decay"
  Attack = (AttackTime == 0.0) ? (0.0) : exp (-1.0 / (SamplingRate * AttackTime));
  Release = (ReleaseTime == 0.0) ? (0.0) : exp (-1.0 / (SamplingRate * ReleaseTime));

  // sample offset to lookahead wnd start
  LookOffset = (int) (SamplingRate * LookAhead);

  // samples count in lookahead window
  LookCount = (int) (SamplingRate * WindowTime);
  */

  
  Target = -20.0f;
  Gate = -50.0f;
  Attacka = 1.0f;
  Attackb = 0.0f;
  Decaya = .001f;
  Decayb = 1.0f - Decaya;
  Nyquist = 22050.0f;
  Level[0] = NGLevel[0] = 0.0f;
  Level[1] = NGLevel[1] = 0.0f;

  NG[0] = 1.0f;
  NG[1] = 1.0f;
  NGI = .01f;
  gain = 1.0f;
  
  Target = (float)(powf(10.f, Target/20.f));
  Gate = (float)(powf(10.f, Gate/20.f));
  MaxGain = Target/Gate; // saves time later
  NG[0] = 0.0f; // variable used to ramp noise gate gain
  NG[1] = 0.0f; // variable used to ramp noise gate gain
  NGI =  500.0f / (Nyquist * 20);  //gatems
  Level[0] = Target; // start fresh
  Level[1] = Target; // start fresh
  NGLevel[0] = 0.0f;
  NGLevel[1] = 0.0f;
  gain = 1.0f;

  float attack = 15.f;
  float decay = 15;

  if (attack == 0.0f)
    { // don't do divide by zero below!
      Attacka = 0.0f;
      Attackb = 1.0f;
    } 
  else
    {// gee, I *think* this time-constant formula is "about" right...
      Attacka = (float)exp((-1000.0/attack)/Nyquist); // number to mul acc by
      Attackb = 1.0f - Attacka; // number to mul input by
    }
  
  Decaya = (float)exp((-1000.0/decay)/Nyquist);
  Decayb = 1.0f - Decaya;
  
  // recently, I added a seperate noisegate level estimator and logic...
  // makes sense to have this act quicker than the normal decay rate
  // used for normal gain-setting...so noise gate can cut on
  // before the gain hits max and brings up the noise floor
  NGa = (float)exp((-1000.0/50.0)/Nyquist); // roughly 50ms decay rate
  NGb = 1.0f - Decaya; 

  }

#define IS_DENORMAL(f) (((*(unsigned int *)&f)&0x7f800000)==0)

void CompressorPlugin::Process(float **input, float **output, long sample_length)
{
  long i;
  int chan;

  /*
  // envelope
  double  env = 0.0;
  int     lki;
  double  smp;
  double  summ;
  double  rms;
  double  theta;
  double  gain;
  
  // for each sample...
  for (int i = 0; i < sample_length; i++)
    {
      // now compute RMS
      summ = 0;
      
      // for each sample in window
      for (int j = 0; j < LookCount; j++)
        {
	  lki = i + j + LookOffset;	  
	  
	  // if we in bounds of signal?
	  // if so, convert to mono
	  if (lki < sample_length)
	    smp = 0.5 * input[0][lki] + 0.5 * input[1][lki];
	  else
	    smp = 0.0;      // if we out of bounds we just get zero in smp
	  
	  summ += smp * smp;  // square em..
        }
      
      rms = sqrt (summ / LookCount);   // root-mean-square
      
      // dynamic selection: attack or release?
      theta = rms > env ? Attack : Release;
      
      // smoothing with capacitor, envelope extraction...
      // here be aware of pIV denormal numbers glitch
      env = (1.0 - theta) * rms + theta * env;
      
      // the very easy hard knee 1:N compressor
      //cout << "env: " << env << "; thres: " << Threshold << endl;
      gain = 1.0;
      if (env > Threshold)
	gain = gain - (env - Threshold) * Slope;

      // result - two hard kneed compressed channels...
      output[0][i] = input[0][i] * gain;
      output[1][i] = input[1][i] * gain;
    }

  */


  
  float ngi2 = NGI * 2.0f;
  union {
    float f;
    unsigned long i;
  } volatile quickabs;
  //Plugin::Process(input, output, sample_length);

  for (chan = 0; chan < 2; chan++)
    for (i = 0; i < sample_length; i++)
      {
	quickabs.f = input[chan][i];    // the future is now!
	quickabs.i &= 0x7fffffff;// maybe 100 times faster than calling fabs...
	// the above just clears the float sign bit to get the abs.
	
	if (quickabs.f > NGLevel[chan]) 
	  NGLevel[chan] = quickabs.f; // instant attack for NG
	else 
	  NGLevel[chan] = NGLevel[chan] * NGa + quickabs.f * NGb; // decay somewhat slower
	
	if (NGLevel[chan] < Gate) // use ng level to decide what to do
	  { // ramp gain multipler downwards linearly
	    if (NG[chan] > 0.0f) 
	      NG[chan] -= NGI;
	    if (NG[chan] < 0.0f) 
	      NG[chan] = 0.0f; // protect from float rounding errors
	  } 
	else 
	  { // coming back on, come up quickly
	    if (NG[chan] < 1.0f) 
	      NG[chan] += ngi2; // gain-up quickerly if above the threshold
	    if (NG[chan] > 1.0f) 
	      NG[chan] = 1.0f;  // just to the right spot, tho
	    
	    // only do level estimation if we're not noise gating -- 
	    // hold last level if we are, but just not doing this
	    
	    if (quickabs.f > Level[chan]) // then do attack filter coeffs
	      {
		Level[chan] = Level[chan] * Attacka + quickabs.f * Attackb;
	      } 
	    else if (NGLevel[chan] > Gate)
	      { // do decay filter coeffs
		Level[chan] = Level[chan] * Decaya + quickabs.f * Decayb;
	      }
	    
	  } // coming back on, or just on  
	gain = ((Target*NG[chan])/Level[chan]);
 
	//  gain = (gain + t2) * 0.5f; // add this line, and the compiler quits working

	// Andrew -- comment out this line and the above to see the fpu bug.
	// all we're doing is averaging current gain and previous gain,
	// but this results often in gain being set to NAN...
	// This code worked until I added a seperate level estimator
	// for use by the noise gate.  I suspect that in some cases
	// the fpu stack isn't getting popped enough after an if
	
	output[chan][i] = input[chan][i] * gain; // actually *do* this job
      } 
  
}

CompressorPlugin::~CompressorPlugin()
{

}

bool CompressorPlugin::IsAudio()
{
  return (true);
}

bool CompressorPlugin::IsMidi()
{
  return (true);
}

wxBitmap *CompressorPlugin::GetBitmap()
{
  return (bmp);
}

void CompressorPlugin::OnTarget(wxScrollEvent &e)
{
  Target = - (100 - TargetFader->GetValue());
  //if (Target <= 0.f)
  //Target = 0.01f;
  cout << "Target: " << Target << "; Res: " << Res << endl;
}
  
void CompressorPlugin::OnResonance(wxScrollEvent &e)
{
  Res = ResFader->GetValue() / 100.f;
  cout << "Target: " << Target << "; Res: " << Res << endl;
}

void CompressorPlugin::OnPaint(wxPaintEvent &event)
{
  wxMemoryDC memDC;
  wxPaintDC dc(this);
  
  memDC.SelectObject(*TpBmp);    
  wxRegionIterator upd(GetUpdateRegion()); // get the update rect list   
  while (upd)
    {    
      dc.Blit(upd.GetX(), upd.GetY(), upd.GetW(), upd.GetH(), &memDC, upd.GetX(), upd.GetY(), 
	      wxCOPY, FALSE);      
      upd++;
    }
}

/******** Main and mandatory library functions *********/

extern "C"
{

  PlugInitInfo init()
  {  
    WIRED_MAKE_STR(info.UniqueId, "COMP");
    info.Name = PLUGIN_NAME;
    info.Type = PLUG_IS_EFFECT;  
    info.UnitsX = 1;
    info.UnitsY = 1;
    return (info);
  }

  Plugin *create(PlugStartInfo *startinfo)
  {
    Plugin *p = new CompressorPlugin(*startinfo, &info);
    return (p);
  }

  void destroy(Plugin *p)
  {
    delete p;
  }

}








